% This ode-function calculates the temperatures in the cylinder.
function dT2=odefun5(t,T2)
global alpha Tinf c dr dz rho A w h hinf Tg a1 a2 aorf L R_N cp_N cv_N gma
global dotmmax r1 r2
dr=0.002;
dz=0.004;
%calculation of piston displacement and velocity
z=A*sin(w*t);
v=w*A*cos(w*t);
%calculate chamber volumes and their derivatives
VA=a1*(L+z);
VB=a2*(L-z);
dotVA=a1*v;
dotVB=-a2*v;
%convert vector with initial values to a matrix to make it easier to match
%it with the geometry.
kmn=0;
T=zeros(26,38);
for j=1:38
T(:,j)=T2((kmn+1):(kmn+26));
kmn=kmn+26;
end
dT=zeros(26,38);
%calculate pressures
pA=T(1,1)*R_N*T(3,1);
pB=T(2,1)*R_N*T(4,1);
p=abs(pA-pB); %pressure difference across seal.
%calculate the leakage
if pA>pB
    Thigh=T(3,1);
    phigh=pA;
    pr=pB/pA;
else
    Thigh=T(3,1);
    phigh=pB;
    pr=pA/pB;
end
dotm=phigh*aorf*(((2*gma/(1-gma))*(1/(R_N*Thigh))*...
    (pr^((gma+1)/gma) - pr^(2/gma)))^0.5);
if dotm >dotmmax 
    dotm=dotmmax;
end
%settting the direction of the flow
if pB>pA
    dotmB=-dotm; %flow exits chamber B
    dotmA=dotm; %flow enters chamber A
    dotmA1=dotmA; %used to calculate enthalpy
    dotmB1=0; %no enthalpy contribution to energy in chamber B
else
    dotmB=dotm; %flow enters chamber B
    dotmA=-dotm; %flow exits chamber A
    dotmA1=0; %no enthalpy contribution to energy in chamber A
    dotmB1=dotmB; %used to calculate enthalpy
end

%calculate the frictional heat (pressure converted to bar)
N=528.5+6.65*p*10^(-5);
mu=0.1498-0.00016*p*10^(-5);
Ff=8.7+0.95*mu*N;
Qpist=Ff*abs(v);
%% determine what node the piston-/rod-seal is at, and what fraction of
% the node surface that is in contact with gas A/B/surrounding.
NR=z/dz;
if z==0
    NR1=0;
    resA=0.5;
    resB=0.5;
elseif z<0
    if abs(NR-floor(NR))>0.5
        NR1=ceil(NR);
        resB=0.5+abs(NR-NR1);
        resA=1-resB;
    elseif abs(NR-floor(NR))<0.5
        NR1=floor(NR);
        resA=0.5+abs(NR-NR1);
        resB=1-resA;
    end
else
    if abs(NR-floor(NR))<0.5
        NR1=floor(NR);
        resA=0.5+abs(NR-NR1);
        resB=1-resA;
    elseif abs(NR-floor(NR))>0.5
        NR1=ceil(NR);
        resB=0.5+abs(NR-NR1);
        resA=1-resB;
    end
end
%% Cylinder and flange
% inner wall surface node
i=9;
f=0;
for j=18:32;
    if j<(25+NR1)
        cent=2;
        Q=0;
    elseif j>(25+NR1)
        cent=1;
        Q=0;
    elseif j==25+NR1
        cent=3;
        Q=Qpist;
    elseif j==26+NR1 && resA==resB 
        cent=1;
        Q=Qpist/2;
    elseif j==24+NR1 && resA==resB
        cent=2;
        Q=Qpist/2;
    end

    dT(i,j)=SurfNodeI(T(i-1,j),T(i,j+1),T(i,j-1),T(i,j),T(3,1),T(4,1),h,i,j,...
        Q,cent,resA,resB);
end

%interior nodes
for i=7:8
    for j=19:33
        dT(i,j)=InteriorNode(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),i,j);
    end
end
for i=2:18
    for j=12:16
        dT(i,j)=InteriorNode(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),i,j);
    end
end
for i=2:8
    for j=17:18
        dT(i,j)=InteriorNode(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),i,j);
    end
end
for i=7:24
    for j=34:37
        dT(i,j)=InteriorNode(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),i,j);
    end
end
j=37;
for i=2:6
        dT(i,j)=InteriorNode(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),i,j);
end
%outer wall surface nodes
i=1;
for j=12:18;    
    dT(i,j)=SurfNode(T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),Tinf,h,i);
end
i=6;
for j=20:35;    
    dT(i,j)=SurfNode(T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),Tinf,h,i);
end
j=37;
i=1;
    dT(i,j)=SurfNode(T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),Tinf,h,i);
%horizontal walls
j=36;
for i=2:5
    dT(i,j)=SurfNodeZ(T(i-1,j),T(i+1,j),T(i,j+1),T(i,j),Tinf,h,i,j);
end
j=33;
for i=10:24
    dT(i,j)=SurfNodeZ(T(i-1,j),T(i+1,j),T(i,j+1),T(i,j),T(3,1),h,i,j);
end
j=19;
for i=2:5
    dT(i,j)=SurfNodeZ(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j),Tinf,h,i,j);
end
j=17;
for i=10:18    
    dT(i,j)=SurfNodeZ(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j),T(4,1),h,i,j);
end

%interior corner nodes outside chamber
j=19;
i=6;
dT(i,j)=IntCornerNodeO(T(i+1,j),T(i-1,j),T(i,j+1),T(i,j-1),T(i,j),Tinf,h,i);
j=36;
i=6;
dT(i,j)=IntCornerNodeO(T(i+1,j),T(i-1,j),T(i,j-1),T(i,j+1),T(i,j),Tinf,h,i);
%interior corner nodes inside chamber
j=17;
i=9;
dT(i,j)=IntCornerNodeI(T(i+1,j),T(i-1,j),T(i,j+1),T(i,j-1),T(i,j),T(4,1),h,i);
j=33;
i=9;
dT(i,j)=IntCornerNodeI(T(i+1,j),T(i-1,j),T(i,j-1),T(i,j+1),T(i,j),T(3,1),h,i);
%exterior corners outside chamber
i=1;
j=36;
    dT(i,j)=ExtCornerNodeO(T(i+1,j),T(i,j+1),T(i,j),Tinf,h,i);
i=1;
j=19;
    dT(i,j)=ExtCornerNodeO(T(i+1,j),T(i,j-1),T(i,j),Tinf,h,i);
% internal nodes at the center of the cylinder
i=25;
for j=34:37
    dT(i,j)=InteriorNode(T(i-1,j),0,T(i,j+1),T(i,j-1),T(i,j),i,j);
end
%surface node center of cylinder
i=25;
j=33;
    dT(i,j)=SurfNodeZ(T(i-1,j),T(i-1,j),T(i,j+1),T(i,j),T(3,1),h,i,j);
%surface node close to flange
i=19;
for j=12:16
    if j==12 || j==13
        cent=1;
    elseif j==15 || j==16
        cent=2;
    else
        cent=3;
    end
    dT(i,j)=SurfNodeI(T(i-1,j),T(i,j+1),T(i,j-1),T(i,j),T(4,1),Tinf,h,i...
        ,j,0,cent,0.5,0.5);
end
%exterior corner node close to flange
i=19;
j=17;
dT(i,j)=ExtCornerNodeI(T(i-1,j),T(i,j-1),T(i,j),T(4,1),h,i);
%The top of the cylinder.
j=11;
for i=2:18
    dT(i,j)=InteriorNode(T(i-1,j),T(i+1,j),T(i,j+1),T(i,j+1),T(i,j),i,j);
end
i=19;
j=11;
    dT(i,j)=SurfNodeI(T(i-1,j),T(i,j+1),T(i,j+1),T(i,j),T(4,1),Tinf,h,i,...
        j,0,1,0,0);
i=1;
j=11;
    dT(i,j)=SurfNode(T(i+1,j),T(i,j+1),T(i,j+1),T(i,j),Tinf,h,i);
%Bottom of the cylinder
j=38;
for i=2:24
    dT(i,j)=InteriorNode(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j-1),T(i,j),i,j);
end
i=25;
j=38;
    dT(i,j)=InteriorNode(T(i-1,j),T(i-1,j),T(i,j-1),T(i,j-1),T(i,j),i,j);
i=1;
j=38;
    dT(i,j)=SurfNode(T(i+1,j),T(i,j-1),T(i,j-1),T(i,j),Tinf,h,i);
    
%% Piston
%Interior nodes
for i=21:25
    for j=2:23
        dT(i,j)=InteriorNodeRod(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),i);
    end
end
for i=11:25
    for j=24:26
        dT(i,j)=InteriorNodeRod(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),i);
    end
end
%top of rod
j=1;
for i=21:25
    dT(i,j)=InteriorNodeRod(T(i-1,j),T(i+1,j),T(i,j+1),T(i,j+1),T(i,j),i);
end
%center of rod
i=26;
for j=2:26
    dT(i,j)=InteriorNodeRod(T(i-1,j),T(i-1,j),T(i,j-1),T(i,j+1),T(i,j),i);
end
%center node at top of the rod
i=26;
j=1;
    dT(i,j)=InteriorNodeRod(T(i-1,j),T(i-1,j),T(i,j+1),T(i,j+1),T(i,j),i);
%nodes at horizontal surfaces
j=27;
for i=11:25
    dT(i,j)=SurfNodeZRod(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j),T(3,1),h,i);
end
j=27;
i=26;
    dT(i,j)=SurfNodeZRod(T(i-1,j),T(i-1,j),T(i,j-1),T(i,j),T(3,1),h,i);
j=23;
for i=11:19
    dT(i,j)=SurfNodeZRod(T(i-1,j),T(i+1,j),T(i,j+1),T(i,j),T(4,1),h,i);
end
%vertical surfaceces
i=10;
for j=24:26
    if j==24
        cent=2;
    elseif j==25
        cent=3;
    elseif j==26
        cent=1;
    end
    dT(i,j)=SurfNodeRod(T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),T(4,1),Tinf...
        ,h,i,0,cent,0.5,0.5);
end
i=20;
for j=2:22 
    if j<(14+NR1)
        cent=2;
        Q=0;
    elseif j>(14+NR1)
        cent=1;
        Q=0;
    elseif j==14+NR1
        cent=3;
        Q=Qpist*r2/r1;
    elseif j==15+NR1 && resA==resB
        Q=Qpist*r2/r1/2;
        cent=1;
    elseif j==13+NR1 && resA==resB
        Q=Qpist*r2/r1/2;
        cent=2;
    end
    dT(i,j)=SurfNodeRod(T(i+1,j),T(i,j-1),T(i,j+1),T(i,j),T(4,1),Tinf...
        ,h,i,Q,cent,resB,resA);
end
i=20;
j=1;
dT(i,j)=SurfNodeRod(T(i+1,j),T(i,j+1),T(i,j+1),T(i,j),Tinf,Tinf,h,i,0,1,0,0);
%exterior corners
i=10;
j=23;
dT(i,j)=ExtCornerNodeORod(T(i+1,j),T(i,j+1),T(i,j),T(4,1),h);
i=10;
j=27;
dT(i,j)=ExtCornerNodeORod(T(i+1,j),T(i,j-1),T(i,j),T(3,1),h);
%interior corner
i=20;
j=23;
dT(i,j)=IntCornerNodeORod(T(i-1,j),T(i+1,j),T(i,j-1),T(i,j+1),T(i,j)...
    ,T(4,1),h,i);
%% Heat transfer from chamber A and B
AVertW=dz*2*pi*r1;
AVertR=dz*2*pi*r2;


Qa=0;
Qb=0;
%heat transfer between gas in chamber A and B to the respective chamber
%walls.
i=9;
for j=18:32
    if j<(25+NR1)
        Qb=Qb+h*AVertW*(T(i,j)-T(4,1));
        
    elseif j>(25+NR1)
        Qa=Qa+h*AVertW*(T(i,j)-T(3,1));
    else
        Qb=Qb+h*AVertW*resB*(T(i,j)-T(4,1));
        Qa=Qa+h*AVertW*resA*(T(i,j)-T(3,1));
    end
end
%corners of the cylinder wall
Qb=Qb+h*AVertW/2*(T(i,17)-T(4,1));
Qa=Qa+h*AVertW/2*(T(i,33)-T(3,1));
%heat transfer between the gas in chamber B and the piston rod
i=20;
for j=(14+NR1):22
    if j>(14+NR1)
        Qb=Qb+h*AVertR*(T(i,j)-T(4,1));
    else
        Qb=Qb+h*AVertR*resA*(T(i,j)-T(4,1));
    end
end
%heat transfer between the gas and corner between rod and piston
Qb=Qb+h*AVertR/2*(T(i,23)-T(4,1));
%heat transfer between the gas and nodes on the vertical piston wall
Qb=Qb+h*AVertW*((T(10,23)-T(4,1))/2+(T(10,24)-T(4,1))+(T(10,25)-T(4,1))/2);
Qa=Qa+h*AVertW*((T(10,27)-T(3,1))/2+(T(10,26)-T(3,1))+(T(10,25)-T(3,1))/2);
%heat transfer between the gas and nodes on the flange wall
Qb=Qb+h*AVertR*((T(19,17)-T(4,1))/2+(T(19,16)-T(4,1))+(T(19,15)-T(4,1))+...
    (T(19,14)-T(4,1))/2);
%heat transfer between the gas and nodes on the top surface of piston
j=23;
for i=10:20
    k=i-9;
    if k==1
        ri=0.063/2;
        ri3=0.063/2-0.001;
        ri2=ri; 
    elseif k==2
        ri2=0.063/2;
        ri=0.063/2-0.001;
        ri3=0.063/2-0.001-dr;
       
    else
        ri2=0.063/2-0.001-(k-3)*dr;
        ri=0.063/2-0.001-(k-2)*dr;
        ri3=0.063/2-0.001-(k-1)*dr;
    end
    if i==20
        ri3=ri;
    end
    ARingR=(((ri2+ri)/2)^2-((ri+ri3)/2)^2)*pi;
    Qb=Qb+h*ARingR*(T(i,j)-T(4,1));
end
%node in corner top surface of piston

%heat transfer between the gas and nodes on the bottom surface of piston
j=27;
for i=10:25
    k=i-9;
    if k==1
        ri=0.063/2;
        ri3=0.063/2-0.001;
        ri2=ri; 
    elseif k==2
        ri2=0.063/2;
        ri=0.063/2-0.001;
        ri3=0.063/2-0.001-dr;
    else
        ri2=0.063/2-0.001-(k-3)*dr;
        ri=0.063/2-0.001-(k-2)*dr;
        ri3=0.063/2-0.001-(k-1)*dr;
    end
    if k==25-9
        ri3=0;
    elseif k==26-9
        ri3=0;
        ri=0;
        ri2=2*ri2;
    end
    ARingR=(((ri2+ri)/2)^2-((ri+ri3)/2)^2)*pi;
    Qa=Qa+h*ARingR*(T(i,j)-T(3,1));
end

%heat transfer between the bottom of the cylinder and the gas
j=33;
for i=9:25
ri=0.095/2-(i-1)*dr;
ri2= 0.095/2-(i-2)*dr;
    if i==24
        ri3=0;
    elseif i==18 && j==17
        ri3=0.0125;
    elseif i==25
        ri=0;
        ri3=0;
        ri2=2*ri2;
    else
        ri3=0.095/2-(i)*dr;
    end
    ARingR=(((ri2+ri)/2)^2-((ri+ri3)/2)^2)*pi;
    Qa=Qa+h*ARingR*(T(i,j)-T(3,1));
end
%heat transfer between the bottom of the flange and the gas
j=17;
for i=9:19
ri=0.095/2-(i-1)*dr;
ri2= 0.095/2-(i-2)*dr;
    if i==18 && j==17
        ri3=0.0125;
    elseif i==19
        ri=2*0.0125;
        ri3=0;
    else
        ri3=0.095/2-(i)*dr;
    end
    ARingR=(((ri2+ri)/2)^2-((ri+ri3)/2)^2)*pi;
    Qb=Qb+h*ARingR*(T(i,j)-T(4,1));
end

%% 
%Density
dT(1,1)=((dotmA-T(1,1)*dotVA)/(VA));
dT(2,1)=((dotmB-T(2,1)*dotVB)/(VB));
%calculate work, enthalpy and internal energy
wA=pA*dotVA;
uA=pA*dotmA/T(1,1);
entA=dotmA1*cp_N*(T(4,1)-T(3,1));
wB=pB*dotVB;
uB=pB*dotmB/T(2,1);
entB=dotmB1*cp_N*(T(3,1)-T(4,1));
%gas temeperatures.
dT(3,1)=(Qa-(wA)+entA-uA)/(T(1,1)*VA*cv_N);
dT(4,1)=(Qb-(wB)+entB-uB)/(T(2,1)*VB*cv_N);

dT2=dT(:,1);
for j=2:38
dT2=[dT2; dT(:,j)];
end


function dT=InteriorNode(T1,T2,T3,T4,T5,i,j)
% This function calculates dT/dt for an interior node at point (i,j)
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha dr dz
ri=0.095/2-(i-1)*dr;
if i==1
    ri2=0;
else
    ri2= 0.095/2-(i-2)*dr;
end
if i==24
    ri3=0;
elseif j>11 && j<17 && i==18
    ri3=0.0125;
else
    ri3=0.095/2-(i)*dr;
end

A1=dz/2*(ri2+ri);
A2=dz/2*(ri+ri3);

A3=((ri2+ri)^2-(ri+ri3)^2)/8;
V=dz*A3;
if i==25
    dr2=0.095/2-23*dr;
    dT=alpha*((2*T1-2*T5)/dr2^2 +(T3-2*T5+T4)/dz^2);
else
    dT=alpha/V*(A1*(T1-T5)/(ri2-ri)+A2*(T2-T5)/(ri-ri3)+A3*(T3-T5)/dz+...
        A3*(T4-T5)/dz);
end
function dT=InteriorNodeRod(T1,T2,T3,T4,T5,i)
% This function calculates dT/dt for an interior node at point (i,j) in the
% rod
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha dr dz
i=i-9;
if i==26-9;
    dr2=0.0025;
    dT=alpha*((2*T1-2*T5)/dr2^2 +(T3-2*T5+T4)/dz^2);
else
    if i==2
        ri2=0.063/2;
        ri=0.063/2-0.001;
        ri3=0.063/2-0.001-dr;
    else
        ri2=0.063/2-0.001-(i-3)*dr;
        ri=0.063/2-0.001-(i-2)*dr;
        ri3=0.063/2-0.001-(i-1)*dr;
    end

    if i==25-9;
        ri3=0;
    end
        dT=(4*alpha)/(((ri2+ri))^2-((ri+ri3))^2)...
        *((ri2+ri)/(ri2-ri)*(T1-T5)+...
        (ri+ri3)/(ri-ri3)*(T2-T5))...
        +alpha/dz^2*((T3-T5)+(T4-T5));
end
function dT=SurfNode(T2,T3,T4,T5,Tinf,h,i)
% This function calculates dT/dt for a node at point (i,j) at a vertical   
% surface subject to convection at the outer radius.
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha c dr dz rho
ri=0.095/2-(i-1)*dr;
if i==24
    ri3=0;
else
ri3=0.095/2-(i)*dr;
end
A1=dz*(ri+ri3)/2;
A2=(4*ri^2-(ri+ri3)^2)/8;
A3=ri*dz;
V=A2*dz;

dT=alpha/V*(A1*(T2-T5)/(ri-ri3)+A2*((T3-T5)/dz+(T4-T5)/dz))+...
    h*A3/(rho*c*V)*(Tinf-T5);
function dT=SurfNodeRod(T2,T3,T4,T5,TgLow,TgTop,h,i,Q,cent,resA,resB)
% This function calculates dT/dt for a node at point (i,j) at a vertical   
% surface subject to convection
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha c dr dz rho
i=i-9;
    if i==1
        ri=0.063/2;
        ri3=0.063/2-0.001;
    elseif i==2
        ri=0.063/2-0.001;
        ri3=0.063/2-0.001-dr;
    else
        ri=0.063/2-0.001-(i-2)*dr;
        ri3=0.063/2-0.001-(i-1)*dr;
    end
A1=dz*(ri+ri3)/2;
A2=(4*ri^2-(ri+ri3)^2)/8;
A3=ri*dz;
V=A2*dz;
if cent==1
    conv=h*A3/(rho*c*V)*(TgLow-T5);
elseif cent==2
    conv=h*A3/(rho*c*V)*(TgTop-T5);
elseif cent==3    
    conv=h*A3/(rho*c*V)*((TgTop-T5)*resA+(TgLow-T5)*resB);
end
dT=alpha/V*(A1*(T2-T5)/(ri-ri3)+A2*((T3-T5)/dz+(T4-T5)/dz))+...
    conv+Q/(2*pi*rho*c*V);
function dT=SurfNodeI(T1,T3,T4,T5,TgLow,TgTop,h,i,j,Q,cent,resA,resB)
% This function calculates dT/dt for a node at point (i,j) at a vertical   
% surface subject to convection at the inner radius.
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha c dr dz rho
if i==1
    ri2=0;
else
    ri2= 0.095/2-(i-2)*dr;
end
if i==19 && j>11 && j<17
    ri=0.0125;
else
   ri=0.095/2-(i-1)*dr; 
end
A1=dz/2*(ri2+ri);
A2=((ri2+ri)^2-4*ri^2)/8;
A3=ri*dz;
V=A2*dz;

if cent==1
    conv=h*A3/(rho*c*V)*(TgLow-T5);
elseif cent==2
    conv=h*A3/(rho*c*V)*(TgTop-T5);
elseif cent==3
    conv=h*A3/(rho*c*V)*(resB*(TgTop-T5)+resA*(TgLow-T5));
end


dT=alpha/V*(A1*(T1-T5)/(ri2-ri)+A2*((T3-T5)/dz+(T4-T5)/dz))+...
    conv+Q/(2*pi*rho*c*V);
function dT=SurfNodeZ(T1,T2,T3,T5,Tinf,h,i,j)
% This function calculates dT/dt for a node at point (i,j) at a horizontal 
% surface subject to convection
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha c dr dz rho
ri=0.095/2-(i-1)*dr;
if i==1
    ri2=0;    
else
    ri2= 0.095/2-(i-2)*dr;
end
if i==24
    ri3=0;
elseif i==18 && j==17
    ri3=0.0125;
else
    ri3=0.095/2-(i)*dr;
end
A1=dz*(ri2+ri)/4;
A2=dz*(ri+ri3)/4;
A3=((ri2+ri)^2-(ri+ri3)^2)/8;
V=dz*A3/2;
if i==25
    dr2=0.095/2-23*dr;
    dT=2*alpha*((T1-T5)/dr2^2+(T3-T5)/dz^2)+2*h/(rho*c*dz)*(Tinf-T5);
else
    dT=alpha/V*(A1*(T1-T5)/(ri2-ri)+A2*(T2-T5)/(ri-ri3)+A3*(T3-T5)/dz)+...
        h*A3*(Tinf-T5)/(rho*c*V);

end
function dT=SurfNodeZRod(T1,T2,T3,T5,Tinf,h,i)
% This function calculates dT/dt for a node at point (i,j) at a horizontal 
% surface subject to convection 
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha c dr dz rho
i=i-9;
if i==26-9
    dr2=0.095/2-23*dr;
    dT=2*alpha*((T1-T5)/dr2^2+(T3-T5)/dz^2)+2*h/(rho*c*dz)*(Tinf-T5);
else
    if i==2
        ri2=0.063/2;
        ri=0.063/2-0.001;
        ri3=0.063/2-0.001-dr;
    else
        ri2=0.063/2-0.001-(i-3)*dr;
        ri=0.063/2-0.001-(i-2)*dr;
        ri3=0.063/2-0.001-(i-1)*dr;
    end
    if i==25-9;
        ri3=0;
    end
A1=dz*(ri2+ri)/4;
A2=dz*(ri+ri3)/4;
A3=((ri2+ri)^2-(ri+ri3)^2)/8;
V=dz*A3/2;
    dT=alpha/V*(A1*(T1-T5)/(ri2-ri)+A2*(T2-T5)/(ri-ri3)+A3*(T3-T5)/dz)+...
        h*A3*(Tinf-T5)/(rho*c*V);
end
function dT=IntCornerNodeO(T1,T2,T3,T4,T5,Tinf,h,i)
% This function calculates dT/dt for a node in a internal corner at 
% point (i,j) 
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha dr dz rho c
ri=0.095/2-(i-1)*dr;
if i==1
    ri2=0;
else
    ri2= 0.095/2-(i-2)*dr;
end
if i==24
    ri3=0;
else
ri3=0.095/2-(i)*dr;
end
A1=dz/4*(ri2+ri);
A2=dz/2*(ri+ri3);
A3=((ri2+ri)^2-(ri+ri3)^2)/8;
A4=(4*ri^2-(ri+ri3)^2)/8;
A5=ri*dz/2;
A6=((ri2+ri)^2-4*ri^2)/8;
V=dz*((ri2+ri)^2-2*(ri+ri3)^2+4*ri^2)/16;

dT=alpha/V*(A1*(T1-T5)/(ri2-ri)+A2*(T2-T5)/(ri-ri3)+A3*(T3-T5)/dz+...
    A4*(T4-T5)/dz)+h*(Tinf-T5)/(rho*c*V)*(A5+A6);
function dT=IntCornerNodeI(T1,T2,T3,T4,T5,Tinf,h,i)
% This function calculates dT/dt for a node at point (i,j) in a 
% internal corner
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha dr dz rho c
ri=0.095/2-(i-1)*dr;
if i==1
    ri2=0;
else
    ri2= 0.095/2-(i-2)*dr;
end
if i==24
    ri3=0;
else
ri3=0.095/2-(i)*dr;
end
V=dz*(2*(ri2+ri)^2-(ri+ri3)^2-4*ri^2)/16;
A1=dz/2*(ri2+ri);
A2=dz/4*(ri+ri3);
A3=((ri2+ri)^2-(ri+ri3)^2)/8;
A4=((ri2+ri)^2-4*ri^2)/8;
A5=ri*dz/2;
A6=(4*ri^2-(ri+ri3)^2)/8;
dT=alpha/V*(A1*(T1-T5)/(ri2-ri)+A2*(T2-T5)/(ri-ri3)+A3*(T3-T5)/dz+...
    A4*(T4-T5)/dz)+h*(Tinf-T5)/(rho*c*V)*(A5+A6);
function dT=ExtCornerNodeO(T2,T3,T5,Tinf,h,i)
% This function calculates dT/dt for a node at an external corner at 
% point (i,j) subject to convection 
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha c rho dz dr 
ri=0.095/2-(i-1)*dr;
if i==24
    ri3=0;
else
ri3=0.095/2-(i)*dr;
end
A1=dz*(ri+ri3)/4;
A2=(4*ri^2-(ri+ri3)^2)/8;
A3=ri*dz/2;
V=A2*dz/2;
dT=alpha/V*(A1*(T2-T5)/(ri-ri3)+A2*(T3-T5)/dz)+h*(Tinf-T5)/(rho*c*V)*...
    (A3+A2);
function dT=ExtCornerNodeORod(T2,T3,T5,Tinf,h)
% This function calculates dT/dt for a node at an external corner at 
% point (i,j) subject to convection 
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha c rho dz
ri=0.063/2;
ri3=0.063/2-0.001;
A1=dz*(ri+ri3)/4;
A2=(4*ri^2-(ri+ri3)^2)/8;
A3=ri*dz/2;
V=A2*dz/2;
dT=alpha/V*(A1*(T2-T5)/(ri-ri3)+A2*(T3-T5)/dz)+h*(Tinf-T5)/(rho*c*V)*...
    (A3+A2);
function dT=ExtCornerNodeI(T1,T3,T5,Tinf,h,i)
% This function calculates dT/dt for a node at an external corner at 
% point (i,j) subject to convection 
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha c rho dz dr 
ri=0.0125;
ri2= 0.095/2-(i-2)*dr;
A1=dz*(ri2+ri)/4;
A2=((ri2+ri)^2-4*ri^2)/8;
A3=ri*dz/2;
V=dz*A2/2;
dT=alpha/V*(A1*(T1-T5)/(ri2-ri)+A2*(T3-T5)/dz)+h*(Tinf-T5)/(rho*V*c)*(A3+A2);
function dT=IntCornerNodeORod(T1,T2,T3,T4,T5,Tinf,h,i)
% This function calculates dT/dt for a node in a internal corner at 
% point (i,j) 
% T1=T(i+1,j), T2=T(i-1,j), T3=T(i,j+1), T4=T(i,j-1), T5=T(i,j), 
% ri=radius i, ri2=radius i+1, ri3=radius i-1 dr=delta r, dz=delta z
global alpha dr dz rho c
i=i-9;
ri2=0.063/2-0.001-(i-3)*dr;
ri=0.063/2-0.001-(i-2)*dr;
ri3=0.063/2-0.001-(i-1)*dr;
A1=dz/4*(ri2+ri);
A2=dz/2*(ri+ri3);
A3=((ri2+ri)^2-(ri+ri3)^2)/8;
A4=(4*ri^2-(ri+ri3)^2)/8;
A5=ri*dz/2;
A6=((ri2+ri)^2-4*ri^2)/8;
V=dz*((ri2+ri)^2-2*(ri+ri3)^2+4*ri^2)/16;
dT=alpha/V*(A1*(T1-T5)/(ri2-ri)+A2*(T2-T5)/(ri-ri3)+A3*(T3-T5)/dz+...
    A4*(T4-T5)/dz)+h*(Tinf-T5)/(rho*c*V)*(A5+A6);