function status = odewbar(t,y,flag,varargin,data)
%ODEWBAR Graphical waitbar printing ODE solver progress.
%   When the function odewbar is passed to an ODE solver as the 'OutputFcn'
%   property, i.e. options = odeset('OutputFcn',@odewbar), the solver calls 
%   ODEWBAR(T,Y,'') after every timestep. The ODEWBAR function shows a
%   waitbar with the progress of the integration every 0.2 seconds.
%   
%   At the start of integration, a solver calls ODEWBAR(TSPAN,Y0,'init') to
%   initialize the output function.  After each integration step to new time
%   point T with solution vector Y the solver calls STATUS = ODEWBAR(T,Y,'').
%	When the integration is complete, the solver calls ODEWBAR([],[],'done').
%
%   See also ODEPLOT, ODEPHAS2, ODEPHAS3, ODE45, ODE15S, ODESET.

%   Jos Pina, 22-11-2006
%	
persistent tfinal hwbar tlast
% regular call -> increment wbar
handles = guidata(gcbo);
if nargin < 3 || isempty(flag)	
	% update only if more than 0.2 sec elapsed
 	if cputime-tlast>0.2
 		tlast = cputime;
        waitbar(t(end)/tfinal,hwbar);
 	% terminate if less than one second elapsed
%  	else
%  		status = 0;
%  		return
    end
    handles.t=[handles.t t];
    handles.rhoA=[handles.rhoA y(1,:)];
    handles.rhoB=[handles.rhoB y(2,:)];
    handles.TB=[handles.TB y(4,:)];
    handles.TA=[handles.TA y(3,:)];
    handles.Ts1=[handles.Ts1 y(937,:)];
    handles.Ts2=[handles.Ts2 y(812,:)];
    handles.Ts3=[handles.Ts3 y(630,:)];
    handles.Ts4=[handles.Ts4 y(313,:)];
    handles.Tend=y(:,end);
if getappdata(hwbar,'canceling');
    status=true;
else
    status=false;
end
% initialization / end
else
  switch(flag)
  case 'init'               % odeprint(tspan,y0,'init')
	hwbar = waitbar(0,'Its hammertime! Du du Du du...','Name','Progress'...
        ,'CreateCancelBtn', 'setappdata(gcbf,''canceling'',1)');
    %,...'WindowStyle','modal');
setappdata(hwbar,'canceling',0);
setappdata(hwbar,'String','Stop. Hammertime')
	tfinal = t(end);
	tlast = cputime;
    if handles.Resume
        handles.t=handles.t;
        handles.rhoA=handles.rhoA;
        handles.rhoB=handles.rhoB;
        handles.TB=handles.TB;
        handles.TA=handles.TA;
        handles.Ts1=handles.Ts1;
        handles.Ts2=handles.Ts2;
        handles.Ts3=handles.Ts3;
        handles.Ts4=handles.Ts4;
    else
        handles.t=t(1);
        handles.rhoA=y(1,:);
        handles.rhoB=y(2,:);
        handles.TB=y(4,:);
        handles.TA=y(3,:);
        handles.Ts1=y(937,:);
        handles.Ts2=y(812,:);
        handles.Ts3=y(630,:);
        handles.Ts4=y(313,:);
    end
    
  case 'done'               % odeprint([],[],'done')
	delete(hwbar)    
  end
end

% if handles.Flag
%         status=true;
% else
%         status = false;
% end


guidata(gcbo, handles);
